* KEYIN.H
* Key values for KEYIN() and INPUT
* Copyright (c) 2011, Ladybridge Systems, All Rights Reserved

$list off

* This include record defines values for two related sets of functions.
*
* KEYIN(), KEYINC() and KEYCODE() return a character, mapping special keys
* to values in the upper half of the 8-bit character set, thus rendering
* the characters at those positions unusable. These key values are also
* used by the INPUTFIELD statement for the value returned via the STATUS()
* function if the input is terminated by use of a reserved character.
*
* KEYINV(), KEYINCV() and KEYCODEV() return the numeric value of the
* character. Special keys are mapped to positions that correspond to the
* Unicode Basic Multilingual Plane Private Use Area. On an ECS mode system,
* this value can be stored as a single character. On a non-ECS mode system,
* it must be processed as a numeric value.

* The two sets of return values for the special keys have the same layout,
* displaced by 63488 (0xF800). This displacement is defined by the token
* KV$DIFF below.


* Values common to both function styles

$define CTRL.A                   1
$define CTRL.B                   2
$define CTRL.C                   3
$define CTRL.D                   4
$define CTRL.E                   5
$define CTRL.F                   6
$define CTRL.G                   7
$define CTRL.H                   8
$define K$BACKSPACE              8
$define KV$BACKSPACE             8
$define CTRL.I                   9
$define K$TAB                    9
$define KV$TAB                   9
$define CTRL.J                  10
$define CTRL.K                  11
$define CTRL.L                  12
$define CTRL.M                  13
$define CTRL.N                  14
$define CTRL.O                  15
$define CTRL.P                  16
$define CTRL.Q                  17
$define CTRL.R                  18
$define CTRL.S                  19
$define CTRL.T                  20
$define CTRL.U                  21
$define CTRL.V                  22
$define CTRL.W                  23
$define CTRL.X                  24
$define CTRL.Y                  25
$define CTRL.Z                  26
$define K$RETURN                13
$define KV$RETURN              0x000D
$define K$ESC                   27
$define KV$ESC                 0x001B
$define K$CTRL.BACKSPACE       127
$define KV$CTRL.BACKSPACE      0x007F
$define K$DEL                  127
$define KV$DEL                 0x007F


* Values for KEYIN(), KEYCODE() and related functions returning a character

$define K$F1                   128
$define K$F2                   129
$define K$F3                   130
$define K$F4                   131
$define K$F5                   132
$define K$F6                   133
$define K$F7                   134
$define K$F8                   135
$define K$F9                   136
$define K$F10                  137
$define K$F11                  138
$define K$F12                  139
$define K$CTRL.F1              140
$define K$CTRL.F2              141
$define K$CTRL.F3              142
$define K$CTRL.F4              143
$define K$CTRL.F5              144
$define K$CTRL.F6              145
$define K$CTRL.F7              146
$define K$CTRL.F8              147
$define K$CTRL.F9              148
$define K$CTRL.F10             149
$define K$CTRL.F11             150
$define K$CTRL.F12             151
$define K$ALT.F1               152
$define K$ALT.F2               153
$define K$ALT.F3               154
$define K$ALT.F4               155
$define K$ALT.F5               156
$define K$ALT.F6               157
$define K$ALT.F7               158
$define K$ALT.F8               159
$define K$ALT.F9               160
$define K$ALT.F10              161
$define K$ALT.F11              162
$define K$ALT.F12              163
$define K$SHIFT.F1             164
$define K$SHIFT.F2             165
$define K$SHIFT.F3             166
$define K$SHIFT.F4             167
$define K$SHIFT.F5             168
$define K$SHIFT.F6             169
$define K$SHIFT.F7             170
$define K$SHIFT.F8             171
$define K$SHIFT.F9             172
$define K$SHIFT.F10            173
$define K$SHIFT.F11            174
$define K$SHIFT.F12            175
$define K$MOUSE                200
$define K$LEFT                 203
$define K$RIGHT                204
$define K$UP                   205
$define K$DOWN                 206
$define K$PAGE.UP              207
$define K$PAGE.DOWN            208
$define K$HOME                 209
$define K$END                  210
$define K$INSERT               211
$define K$DELETE               212
$define K$CTRL.TAB             213
$define K$CTRL.PAGE.UP         214
$define K$CTRL.PAGE.DOWN       215
$define K$CTRL.HOME            216
$define K$CTRL.END             217
$define K$USER0                218
$define K$USER1                219
$define K$USER2                220
$define K$USER3                221
$define K$USER4                222
$define K$USER5                223
$define K$USER6                224
$define K$USER7                225
$define K$USER8                226
$define K$USER9                227

* Values for  KEYINV(), KEYCODEV() and related functions returning a number

$define KV$F1                  0xF880
$define KV$F2                  0xF881
$define KV$F3                  0xF882
$define KV$F4                  0xF883
$define KV$F5                  0xF884
$define KV$F6                  0xF885
$define KV$F7                  0xF886
$define KV$F8                  0xF887
$define KV$F9                  0xF888
$define KV$F10                 0xF889
$define KV$F11                 0xF88A
$define KV$F12                 0xF88B
$define KV$CTRL.F1             0xF88C
$define KV$CTRL.F2             0xF88D
$define KV$CTRL.F3             0xF88E
$define KV$CTRL.F4             0xF88F
$define KV$CTRL.F5             0xF890
$define KV$CTRL.F6             0xF891
$define KV$CTRL.F7             0xF892
$define KV$CTRL.F8             0xF893
$define KV$CTRL.F9             0xF894
$define KV$CTRL.F10            0xF895
$define KV$CTRL.F11            0xF896
$define KV$CTRL.F12            0xF897
$define KV$ALT.F1              0xF898
$define KV$ALT.F2              0xF899
$define KV$ALT.F3              0xF89A
$define KV$ALT.F4              0xF89B
$define KV$ALT.F5              0xF89C
$define KV$ALT.F6              0xF89D
$define KV$ALT.F7              0xF89E
$define KV$ALT.F8              0xF89F
$define KV$ALT.F9              0xF8A0
$define KV$ALT.F10             0xF8A1
$define KV$ALT.F11             0xF8A2
$define KV$ALT.F12             0xF8A3
$define KV$SHIFT.F1            0xF8A4
$define KV$SHIFT.F2            0xF8A5
$define KV$SHIFT.F3            0xF8A6
$define KV$SHIFT.F4            0xF8A7
$define KV$SHIFT.F5            0xF8A8
$define KV$SHIFT.F6            0xF8A9
$define KV$SHIFT.F7            0xF8AA
$define KV$SHIFT.F8            0xF8AB
$define KV$SHIFT.F9            0xF8AC
$define KV$SHIFT.F10           0xF8AD
$define KV$SHIFT.F11           0xF8AE
$define KV$SHIFT.F12           0xF8AF
$define KV$MOUSE               0xF8C8
$define KV$LEFT                0xF8CB
$define KV$RIGHT               0xF8CC
$define KV$UP                  0xF8CD
$define KV$DOWN                0xF8CE
$define KV$PAGE.UP             0xF8CF
$define KV$PAGE.DOWN           0xF8D0
$define KV$HOME                0xF8D1
$define KV$END                 0xF8D2
$define KV$INSERT              0xF8D3
$define KV$DELETE              0xF8D4
$define KV$CTRL.TAB            0xF8D5
$define KV$CTRL.PAGE.UP        0xF8D6
$define KV$CTRL.PAGE.DOWN      0xF8D7
$define KV$CTRL.HOME           0xF8D8
$define KV$CTRL.END            0xF8D9
$define KV$USER0               0xF8DA
$define KV$USER1               0xF8DB
$define KV$USER2               0xF8DC
$define KV$USER3               0xF8DD
$define KV$USER4               0xF8DE
$define KV$USER5               0xF8DF
$define KV$USER6               0xF8E0
$define KV$USER7               0xF8E1
$define KV$USER8               0xF8E2
$define KV$USER9               0xF8E3


$define KV$LO.KEY              0xF880
$define KV$DIFF                0xF800
$define KV$HI.KEY              0xF8E3
